from .abuse_requests import AbuseRequestEndpoint
from .achievements import AchievementsEndpoint
from .animes import AnimeEndpoint
from .appears import AppearsEndpoint
from .bans import BanEndpoint
from .base import BaseEndpoint
from .calendar import CalendarEndpoint
from .characters import CharacterEndpoint
from .club import ClubEndpoint
from .comments import CommentEndpoint
from .constants import ConstantsEndpoint
from .dialogs import DialogsEndpoint
from .episode_notifications import EpisodeNotificationEndpoint
from .favorites import FavoritesEndpoint
from .forums import ForumEndpoint
from .friends import FriendEndpoint
from .genres import GenreEndpoint
from .manga import MangaEndpoint
from .message import MessageEndpoint
from .people import PeopleEndpoint
from .publisher import PublisherEndpoint
from .ranobe import RanobeEndpoint
from .review import ReviewEndpoint
from .stats import StatsEndpoint
from .studios import StudiosEndpoint
from .styles import StylesEndpoint
from .topic_ignore import TopicIgnoreEndpoint
from .topics import TopicsEndpoint
from .user_ignore import UserIgnoreEndpoint
from .user_images import UserImageEndpoint
from .user_rates import UserRatesEndpoint
from .users import UserEndpoint
from .videos import VideosEndpoint
from .grapql import GraphQlEndpoint

__all__ = [
    "AbuseRequestEndpoint",
    "GraphQlEndpoint",
    "AchievementsEndpoint",
    "AnimeEndpoint",
    "AppearsEndpoint",
    "BanEndpoint",
    "BaseEndpoint",
    "CalendarEndpoint",
    "CharacterEndpoint",
    "ClubEndpoint",
    "CommentEndpoint",
    "ConstantsEndpoint",
    "DialogsEndpoint",
    "EpisodeNotificationEndpoint",
    "FavoritesEndpoint",
    "ForumEndpoint",
    "FriendEndpoint",
    "GenreEndpoint",
    "MangaEndpoint",
    "MessageEndpoint",
    "PeopleEndpoint",
    "PublisherEndpoint",
    "RanobeEndpoint",
    "ReviewEndpoint",
    "StatsEndpoint",
    "StudiosEndpoint",
    "StylesEndpoint",
    "TopicIgnoreEndpoint",
    "TopicsEndpoint",
    "UserIgnoreEndpoint",
    "UserImageEndpoint",
    "UserRatesEndpoint",
    "UserEndpoint",
    "VideosEndpoint",
]
